/* xpyre/main.c: Main thread
 *
 * $Id: main.c,v 1.6 2002/12/08 02:26:08 andy Exp $
 *
 * Copyright (C) 2002 Andy Goth <unununium@openverse.com>
 * For more information visit http://ioioio.net/devel/xpyre/
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA  02111-1307, USA. */

#include <allegro.h>
#include <errno.h>
#include <pthread.h>
#include <stdio.h>
#include <time.h>
#include "xpyre.h"

/* Prototypes. */
static void* main_loop (void* arg);

static int       main_thread_running;
static pthread_t main_thread;

/* Starts the main thread. */
void main_start(void)
{
    pthread_create(&main_thread, NULL, main_loop, NULL);
    main_thread_running = 1;
}

/* Stops the main thread.  Note that this function can be called within an
 * xmms thread if xmms shuts down or otherwise wants to disable xpyre.  But
 * it can also be called from an xpyre thread if xpyre calls xmms's
 * disable_plugin function.  So I dodge this bug by simply not ever having
 * xpyre request its own termination. */
void main_stop(void)
{
    if (main_thread_running) {
        main_thread_running = 0;
        pthread_cancel(main_thread);
        pthread_join(main_thread, NULL);
    }
}

/* The main thread function. */
void* main_loop(void* arg)
{
    /* TODO: make frame_cap configurable.  Right now it's dependent on the
     * draw code internally using two display buffers. */
    const int frame_cap = 2;            /* Total number of frame buffers. */
    int       frame_num = -1;           /* (Number of ready frames) - 1.  */
    BITMAP*   frame_bmp[frame_cap];     /* Pointers to frame BITMAP's.    */

    pthread_setcancelstate(PTHREAD_CANCEL_ENABLE, NULL);
    pthread_setcanceltype(PTHREAD_CANCEL_ASYNCHRONOUS, NULL);

    /* Here it all is...  ideas for improvement? */
    while (1) {
        if (timer_val == 0) {
            /* Ahead of schedule... */

            if (frame_num < frame_cap - 1) {
                /* There is at least one empty frame slot. */

                /* Fill it. */
                frame_bmp[++frame_num] = draw_frame();

                /* Check to see if we're still ahead of schedule. */
                continue;
            } else {
                /* No more double buffer slots--- forced to wait. */

                /* Wait for timer_val to become nonzero. */
                pthread_mutex_lock(&timer_mutex);
                while (timer_val == 0)
                    pthread_cond_wait(&timer_cond, &timer_mutex);
                pthread_mutex_unlock(&timer_mutex);

                /* Fall through to the timer-has-ticked code. */
            }
        }

        /* Okay, there is at least one unprocessed timer tick. */

        if (frame_num == -1) {
            /* There are no frames available to display!  Fix the situation. */
            frame_bmp[++frame_num] = draw_frame();
        }

        /* Now there must be at least one frame ready to display. */

        /* Update the screen.  TODO: avoid the one-pixel black border. */
        blit(frame_bmp[frame_num], screen, 1, 0, 1, 1,
                SCREEN_W - 2, SCREEN_H - 2);

        /* Discard the displayed frame.  If we were ahead but are now behind,
         * drop more frames to avoid forever displaying old stuff. */
        frame_num = MAX(frame_num - timer_val, -1);

        /* Reset the timer.  Note that the above line processed all ticks by
         * skipping the appropriate number of frames.  TODO: do a better job
         * of fast-forwarding display logic.  TODO: try to make logic update at
         * a speed independent of the fps rate. */
        pthread_mutex_lock(&timer_mutex);
        timer_val = 0;
        pthread_mutex_unlock(&timer_mutex);

        /* Handle the keyboard.  This is done semi-regularly but it's not
         * important to get it every frame since Allegro has a handy keyboard
         * buffer for us to use. */
        if (keypressed()) {
            switch (readkey() >> 8) {
            case KEY_ESC:
                config.fullscreen = !config.fullscreen;
                draw_set_gfx_mode();
                break;
            case KEY_Z: xmms_prior(); break;
            case KEY_X: xmms_play();  break;
            case KEY_C: xmms_pause(); break;
            case KEY_V: xmms_stop();  break;
            case KEY_B: xmms_next();  break;
            case KEY_T: draw_show_title();
            }
        }

        /* Check for song change.  Again, this doesn't need to be done
         * absolutely all the time, since no one will notice if the track
         * change display is a frame or two late.  In fact, it's probably
         * done much too frequently as it is. */
        if (config.draw_title && xmms_track_info())
            draw_show_title();
    }

    /* This never happens. */
    return NULL;
}

/* This junk appears to fool Allegro into working... */
int _mangled_main(int argc, char** argv)
{
    printf("Moo.\n");
    return EXIT_SUCCESS;
}
void* _mangled_main_address = _mangled_main;

/* vim: set ts=4 sts=4 sw=4 tw=80 et: */
/* EOF */
