/* xpyre/timer.c: xpyre timer thread.
 *
 * $Id: timer.c,v 1.2 2002/11/08 03:39:48 andy Exp $
 *
 * Copyright (C) 2002 Andy Goth <unununium@openverse.com>
 * For more information visit http://ioioio.net/devel/xpyre/
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA  02111-1307, USA. */

#include <pthread.h>
#include "xpyre.h"

volatile int    timer_val;
pthread_mutex_t timer_mutex;
pthread_cond_t  timer_cond;

static unsigned  timer_delay;
static int       timer_thread_running;
static pthread_t timer_thread;

static void* timer_loop(void* arg);

/* Starts the timer thread. */
void timer_start(int delay)
{
    timer_val = 0;
    timer_delay = delay;
    pthread_mutex_init(&timer_mutex, NULL);
    pthread_cond_init(&timer_cond, NULL);
    pthread_create(&timer_thread, NULL, timer_loop, NULL);
    timer_thread_running = 1;
}

/* Stops the timer thread. */
void timer_stop(void)
{
    if (timer_thread_running) {
        timer_thread_running = 0;
        pthread_cancel(timer_thread);
        pthread_join(timer_thread, NULL);
        pthread_mutex_destroy(&timer_mutex);
        pthread_cond_destroy(&timer_cond);
    }
}

/* Timer thread function. */
static void* timer_loop(void* arg)
{
    /* TODO: multiplex graphics and logic timing. */

    pthread_setcancelstate(PTHREAD_CANCEL_ENABLE, NULL);
    pthread_setcanceltype(PTHREAD_CANCEL_ASYNCHRONOUS, NULL);

    /* Loop until cancelled by timer_stop(). */
    while (1) {
        pthread_mutex_lock(&timer_mutex);
        ++timer_val;
        pthread_cond_signal(&timer_cond);
        pthread_mutex_unlock(&timer_mutex);

        /* I know, this is not precise.  If pthread_mutex_lock() or
         * pthread_cond_signal() block for any amount of time, the timer will
         * be slowed down.  But it's close enough for a graphics hack. */
        usleep(timer_delay);
    }

    /* Doesn't happen. */
    return NULL;
}

/* vim: set ts=4 sts=4 sw=4 tw=80 et: */
/* EOF */
