/* xpyre/util.c: Useful functions not specific to xpyre.
 *
 * $Id: util.c,v 1.1 2002/12/08 02:26:33 andy Exp $
 *
 * Copyright (C) 2002 Andy Goth <unununium@openverse.com>
 * For more information visit http://ioioio.net/devel/xpyre/
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA  02111-1307, USA. */

#include <allegro.h>
#include <string.h>
#include "xpyre.h"

/* Returns a random number in the range 0 to (2 ** 'bits' - 1). */
int my_rand(int bits)
{
    static unsigned mask[] = {
        0x00000000, 0x00000001, 0x00000003, 0x00000007,
        0x0000000f, 0x0000001f, 0x0000003f, 0x0000007f,
        0x000000ff, 0x000001ff, 0x000003ff, 0x000007ff,
        0x00000fff, 0x00001fff, 0x00003fff, 0x00007fff,
        0x0000ffff, 0x0001ffff, 0x0003ffff, 0x0007ffff,
        0x000fffff, 0x001fffff, 0x003fffff, 0x007fffff,
        0x00ffffff, 0x01ffffff, 0x03ffffff, 0x07ffffff,
        0x0fffffff, 0x1fffffff, 0x3fffffff, 0x7fffffff, 0xffffffff
    };
    static unsigned val = 0;
    static int top = 33;

    if (top + bits <= 32) {
        return val >> (top += bits) & mask[bits];
    } else if (top != 32) {
        top = 32 - top;
        bits -= top;
        return (val >> top & mask[top]) << bits |
                ((val = rand()) & mask[top = bits]);
    } else {
        val = rand();
        top = bits;
        return val & mask[bits];
    }
}

/* Converts 'text', a string in the format "rrggbb" to an Allegro color value,
 * in packed rgb byte ordering, to be corrected later.  A # at the beginning of
 * 'text' is optional.  If there is an error in 'text', 'def' is returned. */
int rrggbb_to_color(char* text, int def)
{
    char buf[] = {0, 0, 0};
    int r, g, b;

    if (strlen(text) == 7 && text[0] == '#') text++;
    if (strlen(text) != 6) return def;

    buf[0] = text[0]; buf[1] = text[1]; r = strtol(buf, NULL, 16);
    buf[0] = text[2]; buf[1] = text[3]; g = strtol(buf, NULL, 16);
    buf[0] = text[4]; buf[1] = text[5]; b = strtol(buf, NULL, 16);

    return r << 16 | g << 8 | b << 0;
}

/* vim: set ts=4 sts=4 sw=4 tw=80 et: */
/* EOF */
