/* xpyre/xpyre.h: Common xpyre header file.
 *
 * $Id: xpyre.h,v 1.4 2002/12/08 02:26:08 andy Exp $
 *
 * Copyright (C) 2002 Andy Goth <unununium@openverse.com>
 * For more information visit http://ioioio.net/devel/xpyre/
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
 * more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 * Place - Suite 330, Boston, MA  02111-1307, USA. */

#ifndef SEEN_XPYRE_H
#define SEEN_XPYRE_H

#include <allegro.h>
#include <glib.h>
#include <pthread.h>
#include <xmms/plugin.h>

/* Screen settings and bitmaps. */
typedef struct config_t {
    gint     width;         /* Width of the screen. */
    gint     height;        /* Height of the screen. */
    gboolean fullscreen;    /* Fullscreen flag. */
    gint     fps;           /* Target frames per second. */
    gboolean draw_title;    /* Draw the track title? */
    int      bg_color;      /* Window background color. */
    gboolean draw_top;      /* Draw the top of the pcm curve? */
    gboolean random_top;    /* Randomize the top? */
    gboolean draw_fill;     /* Fill the pcm curve? */
    gboolean random_fill;   /* Randomize the fill? */
} config_t;

/* Information about a track in the playlist. */
typedef struct track_t {
    int   num;              /* Number position in playlist. */
    char* title;            /* Title of track. */
    int   title_cap;        /* Capacity of title buffer. */
    int   length;           /* Length of track in seconds. */
} track_t;

/* xpyre/xmms.c */
extern pthread_mutex_t pcm_data_mutex;
extern gint16          pcm_data[512];
extern config_t        config;
extern track_t         track;
extern VisPlugin* get_vplugin_info (void);
extern void xmms_disable    (void);
extern void xmms_prior      (void);
extern void xmms_play       (void);
extern void xmms_pause      (void);
extern void xmms_stop       (void);
extern void xmms_next       (void);
extern int  xmms_track_info (void);

/* xpyre/main.c */
extern void* _mangled_main_address;
extern void  main_start    (void);
extern void  main_stop     (void);
extern int   _mangled_main (int argc, char** argv);

/* xpyre/draw.c */
extern int     draw_start        (void);
extern void    draw_stop         (void);
extern int     draw_set_gfx_mode (void);
extern void    draw_show_title   (void);
extern BITMAP* draw_frame        (void);

/* xpyre/timer.c */
extern volatile int timer_val;
extern pthread_mutex_t timer_mutex;
extern pthread_cond_t timer_cond;
extern void timer_start(int delay);
extern void timer_stop(void);

/* xpyre/util.c */
extern int my_rand(int bits);
extern int rrggbb_to_color(char* text, int def);

#endif

/* vim: set ts=4 sts=4 sw=4 tw=80 et: */
/* EOF */
